/***************************************************************************************************/
/*                                                                                                 */
/* file:          BalancedPT.ino                                                                   */
/*                                                                                                 */
/* source:        2015-2106, written by Adrian Kundert                                             */
/*                                                                                                 */
/* description:   IMU sensor MPU-6050 I2C device class                                             */
/*                Based on the I2Cdev library collection by Jeff Rowberg <jeff@rowberg.net>        */
/*                and Gyro_header lib from Omer Ikram ul Haq (http://hobbylogs.me.pn/?p=47)        */
/***************************************************************************************************/

#include <Wire.h>

#include "I2Cdev.h"         // I2Cdev must be installed as library, or else the .cpp/.h files
#include "Controller.h"

Controller Ctrl;

// ================================================================
// ===                    ARDUINO INITIAL SETUP                 ===
// ================================================================
void setup() {
    
    // initialize serial communication to sabertooth default
    Serial.begin(9600);
    // increase the sabertooth baudrate communication
    Serial.write(128); // addr
    Serial.write(15);  // baudrate command
    Serial.write(4);   // 38400 baud
    Serial.write((128+15+4)&0x7f);
    delay(500);
    // re-initialize serial communications at 38400 bps:
    Serial.end();
    Serial.begin(38400);
    delay(1000);
    
    Serial.print("Sw Version:"); Serial.println(SwVersion);
    
    // join I2C bus (I2Cdev library doesn't do this automatically)
    Wire.begin();
    TWBR = 24; // 400kHz I2C clock (200kHz if CPU is 8MHz)
    Serial.println(F("400kHz I2C clock ..."));

    // enable Arduino interrupt detection from mpuTilt
    Serial.println(F("Enabling interrupt detection (Arduino external interrupt 0)..."));
    attachInterrupt(0, dmpDataReady, RISING);

    // configure the IOs
    pinMode(LED_PIN, OUTPUT);
    pinMode(READY_SW_PIN, INPUT);
    pinMode(LOADED_PIN, INPUT);

    // configure the whole controller
    Ctrl.initialize();
}

// ================================================================
// ===                    MAIN PROGRAM LOOP                     ===
// ================================================================
void loop() {
    static bool blinkState = false;
    static unsigned int blinkCnt = 0;
    
    // main process
    Ctrl.process();
    
    // sequenced serial output (a single char a function call to avoid blocking execution)
    Ctrl.serialPrint();
    
    // activity monitor (the led blinking will slow with increasing process() load)
    if(++blinkCnt >= 5000) {
        // blink LED to indicate activity
        blinkState = !blinkState;
        digitalWrite(LED_PIN, blinkState);
        blinkCnt = 0;
    }
}

// ================================================================
// ===               INTERRUPT DETECTION ROUTINE                ===
// ================================================================
void dmpDataReady() {

    interrupts();
    Ctrl.MPUinterrupt();
    noInterrupts();
}







